#!/usr/bin/perl

# cvsexec.pl - Backgrounding script for CVS Monitor
# Copyright (C) 2002-2004 Adam Kennedy
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Should you wish to utilise this software under a different licence, 
# please contact the author.

# Set up the Perl core environment
require 5.005;
use strict;
# Warnings and diagnostics are turned off for production
# use warnings; 
# use diagnostics;
use UNIVERSAL 'isa';

# CVSMonitor does NOT work with mod_perl. To prevent people from
# trying ( THAT MEANS YOU ), die if we detect it. Don't try to make
# it work, if it was easy, it would have been done already.
BEGIN {
	if ( $ENV{MOD_PERL} ) {
		die "CVSMonitor will not work under mod_perl. "
		. "Please execute as a normal CGI script.";
	}
}

# Always run from the directory the script is in
use FindBin; BEGIN { chdir $FindBin::Bin }

# Specify the location of the other libraries
BEGIN {
	@INC = (
		'modules',
		'../../modules',
		'../../../AppSpace2/cgi-bin/Classes',
		@INC,
		'bundled'
		);
}

# Load in the modules
use Class::Autouse qw{:devel};
use Getopt::Long;
use File::Spec;
use File::Flat;
use CVSMonitor;
use CVSMonitor::MetaData;

# Initialise globals and modules
use vars qw{$MetaData $Module};
use vars qw{$verbose $errstr $Config $logging};
use vars qw{$logfile $command_log};
BEGIN {
	# Turn of line buffering
	$| = 1;

	$verbose = '';
	$errstr = '';
	$logfile = '';
	$command_log = '';

	# Make sure the Config exists
	$Config = CVSMonitor::Config->load( './main.conf' );
	unless ( $Config ) {
		# Refuse to run
		die( "Failed to load config file. " . CVSMonitor::Config->errstr );
	}
}

# Use Getopt::Long to get the program options
GetOptions( 'verbose|v' => \$verbose, 'log|l' => \$logging );

# Set the commnand ( for use in logging ) now the options have been removed
$command_log = join ' ', @ARGV;

# Check the work area
my $root = $Config->getHostScratch;
my $base = CVSMonitor::MetaData->_checkWorkArea( $root )
        or die "Error checking work area";
my $file = File::Spec->catfile( $base, 'index.dat' )
        or die "Error determining index file name";
die "MetaData index does not exist" unless -e $file;

# Initialise the work area
unless ( $MetaData = CVSMonitor::MetaData->load( $root, 'lock' ) ) {
        die CVSMonitor::MetaData->errstr;
}

# Turn on logging if requested
if ( $logging ) {
	# Establish the name of the log file, based on the pid of this process
	$logfile = $MetaData->pathAdminLogFile;

	# Print a message telling the console user where we are logging to
	print "Logging to $logfile...\n" if $verbose;

	# Make sure we can write to the file
	unless ( File::Flat->canWrite( $logfile ) ) {
		die 'Insufficient permissions to write to log file';
	}

	# Make sure the directory is there
	File::Flat->_ensureDirectory( $logfile );

	# Open a handle to the file
	open( LOGFILE, ">$logfile" ) or die 'Failed to open log file for writing';

	# Connect STDOUT and STDERR to the log file
	*STDOUT = *LOGFILE;
	*STDERR = *LOGFILE;

	# Add some initial lines
	message( "Command - $command_log" );
}

# Get the arguments and do the action
my $command = shift @ARGV;
Error( "No command given" ) unless $command;
if ( $command eq 'list' ) {
	list();
} elsif ( $command eq 'checkout' ) {
	my $module = shift @ARGV;
	module_command( $module, 'checkout' );
	module_command( $module, 'update' ); 

} elsif ( $command eq 'update' ) {
	module_command( shift(@ARGV), 'update' );

} elsif ( $command eq 'updaterepository' ) {
	repository_command( shift(@ARGV), 'update' );

} elsif ( $command eq 'updateall' ) {
	update_all();

} elsif ( $command eq 'delete' ) {
	module_delete( shift(@ARGV) );

} else {
	Error( "Unknown command '$command'" );
}

# Release the metadata lock
$MetaData->releaseLock;

exit(0);





#####################################################################
# Command subs

# List the modules available
sub list {
	verbose( "List of All Modules" );
	foreach ( sort map { $_->name } $MetaData->getModules ) {
		message( $_ );
	}
	1;
}

sub module_command {
	my $name = shift;
	my $method = shift;

	# Get the Module
	Error( "You did not specify a module to $method" ) unless $name;
	my $Module = $MetaData->getModule( $name );
	Error( "The module '$name' does not exist" ) unless $Module;

	verbose( $method eq 'update'
		? "Updating module $name"
		: "Checking out module $name" );

	if ( $logfile ) {
		# Release our handle on the log file
		close LOGFILE;
	}

	# Do the action on the module
	my $rv = $Module->$method( $logfile );

	if ( $logfile ) {
		# Reopen the log file
        	open( LOGFILE, ">>$logfile" ) or die 'Failed to open log file for writing';
        	*STDOUT = *LOGFILE;
        	*STDERR = *LOGFILE;
	}

	unless ( $rv ) {
                Error( "Error during $method: " . $MetaData->errstr );
	}

	verbose( $method eq 'update'
		? "Module $name updated"
		: "Module $name checked out" );

	# Save and don't release the lock on the index
	$MetaData->save( 'lock' );
	verbose( 'Saved MetaData to scratch area' );

	1;
}

sub repository_command {
	my $name = shift;
	my $method = shift;

	# Get the Module
	Error( "You did not specify a module to $method" ) unless $name;
	my $Repository = $MetaData->getRepository( $name );
	Error( "The module '$name' does not exist" ) unless $Repository;

	# Do the action on the module
	verbose( "Updating repository $name" );
	unless ( $Repository->$method() ) {
		Error( "Error during $method: " . $MetaData->errstr );
	}
	verbose( "Repository $name updated" );

	# Save and release the lock on the index
	$MetaData->save;

	1;
}

sub module_delete {
	my $name = shift;

	# Get the Module
	Error( "You did not specify a module to delete" ) unless $name;
	my $Module = $MetaData->getModule( $name );
	Error( "The module '$name' does not exist or has already been deleted" ) unless $Module;

	# Delete the module
        verbose( "Deleting module $name" );
	unless ( $MetaData->deleteModule( $name ) ) {
		Error( "Error while deleting: " . $MetaData->errstr );
	}
	verbose( "Module $name deleted" );

	# Save the index and reload the view
	$MetaData->save;
}

# Update all the modules 
sub update_all {
	my @Modules = $MetaData->getModules;

	verbose( "Updating all modules" );

	# Iterate over the updatable modules and update them
	foreach my $Module ( sort grep { $_->{state} eq 'Available' } @Modules ) {
		# Do the action on the module.
		# We don't care if it fails.
		verbose( "Updating module " . $Module->display . "..." );
		if ( $Module->update ) {
			verbose( 'Module ' . $Module->display . 'updated.' );
		} else {
			verbose( 'Failed to update ' . $Module->display );
			verbose( split /\:/, $MetaData->errstr );
		}

		# Reload the index
		$MetaData->reload or Error( "Error reloading index after update" );
	}

	# Save the index file
	verbose( "Complete, saving index..." );
	$MetaData->save;
	verbose( 'Update all modules completed' );
}





#####################################################################
# Utility Functions

# Handle errors
sub Error {
	foreach ( @_ ){
		message( $_ );
	} 
	exit(1);
}

sub _error {
	$errstr = shift;
	undef;
}

sub message {
	foreach ( @_ ) {
		my $message = $_;
		chomp $message;
		$message =~ s/:/:\n    /g;
		foreach my $line ( split /\n/, $message ) {
			if ( $logging ) {
				print scalar localtime time;
				print ' ';
			}
			print $line . "\n";
		}
	}
	1;
}

sub verbose {
	$verbose ? message( @_ ) : 1;
}
