package AppLib::CGI;

# The AppLib::CGI module provides a wrapper for CGI.pm.
# It avoids some limitations of CGI.pm.
# File upload parameters will ONLY work for the first CGI object
# created. Hence, this module caches the CGI object, instead of creating
# new ones.

use strict;
use UNIVERSAL 'isa';
use base 'AppLib::Error';
use CGI qw{-debug};

use vars qw{$cache $cache_id};
BEGIN {
	$cache    = undef;
	$cache_id = undef;
}

# If the apache server does not have mod_unique_id installed,
# you will need to call this directly at the beginning of each call.
sub clear_cache {
	$cache = undef;
	$cache_id = undef;
}

sub new {
	my $class = shift;

	# Can we expire the cache
	if ( $cache and $cache_id and $ENV{UNIQUE_ID} ne $cache_id ) {
		$cache = undef;
	}

	# Get the query if we don't have it already, or if we had
	# to expire it.
	unless ( $cache ) {
		$cache = CGI->new(@_);
		$cache_id = $ENV{UNIQUE_ID};
	}

	$cache;
}

# Pseudo emulation of the ReadParse function
# Returns a non-tied hash
use vars qw{%in};
sub ReadParse {
	my $class = shift;
	my $query = $class->new;

	# Build the hash
	my %hash = ();
	foreach ( $query->param ) {
		$hash{$_} = $query->param( $_ ) unless exists $hash{$_};
	}

	\%hash;
}

# The scan method will scan a CGI query to determine a set of names
# within a certain namespace
# For example. Given a params of "a", "a_b", and "b", AppLib::CGI->scan would
# return ( "a", "b" ) and AppLib::CGI->scan( 'a' ) would return 'b'
sub scan {
	my $class = shift;
	my $query = shift;
	my $namespace = shift;
	$query = $class->new unless $query;
	$namespace .= '_' if $namespace ne '';

	# Get the params list
	my @params = $query->param;

	# Go through the list and build the list
	my %names = ();
	foreach ( @params ) {
		if ( $_ =~ /^$namespace([a-zA-Z0-9]+)_?/ ) {
			$names{$1} = 1;
		}
	}

	sort keys %names;
}

# Move an uploaded file to somewhere else
sub saveUpload {
	my $class = shift;
	my $name = shift;
	my $outfile = shift;

	# Get a handle
	my $query = $class->new;
	my $filename = $query->param( $name );
	return 0 unless $filename;

	# Copy a binary file to somewhere else
	my $rv = open ( OUTFILE, ">>$outfile" );
	return $class->andError( "Error opening output file '$outfile'" ) unless $rv;
	binmode OUTFILE;
	my ( $buffer, $bytesread );
	while ( $bytesread = read( $filename, $buffer, 1024 ) ) {
		print OUTFILE $buffer;
	}
	close OUTFILE;
	1;
}

# Save a CGI request to a file
sub save {
	my $class = shift;
	my $filename = shift;
	my $query = $class->new;

	open( OUTFILE, ">$filename" ) or return undef;
	{
		no strict;
		$query->save( OUTFILE );
	}
	close OUTFILE;
	1;
}

sub header {
	# Pass through
	my $class = shift;
	CGI::header(@_);
}

1;
